@@ -1,3 +1,5 @@
+from __future__ import unicode_literals
+
 import os
 
 import pytest
@@ -5,7 +7,7 @@ import pytest
 from bleach import clean
 from bleach.html5lib_shim import Filter
 from bleach.sanitizer import Cleaner
-
+from bleach._vendor.html5lib.constants import rcdataElements
 
 def test_clean_idempotent():
     """Make sure that applying the filter twice doesn't change anything."""
@@ -22,11 +24,11 @@ def test_only_text_is_cleaned():
 
     with pytest.raises(TypeError) as e:
         clean(some_type)
-    assert "argument cannot be of 'type' type" in str(e)
+    assert "argument cannot be of 'type' type" in str(e.value)
 
     with pytest.raises(TypeError) as e:
         clean(no_type)
-    assert "NoneType" in str(e)
+    assert "NoneType" in str(e.value)
 
 
 def test_empty():
@@ -218,7 +220,7 @@ def test_bare_entities_get_escaped_correctly(text, expected):
     ('this &amp that', 'this &amp;amp that'),
 
     # Test a thing that looks like a character entity, but isn't because it's
-    # missing a ; (&curren)
+    # missing a ; (&current)
     (
         'http://example.com?active=true&current=true',
         'http://example.com?active=true&amp;current=true'
@@ -471,10 +473,10 @@ def test_attributes_callable():
     ATTRS = lambda tag, name, val: name == 'title'
     TAGS = ['a']
 
-    text = u'<a href="/foo" title="blah">example</a>'
+    text = '<a href="/foo" title="blah">example</a>'
     assert (
         clean(text, tags=TAGS, attributes=ATTRS) ==
-        u'<a title="blah">example</a>'
+        '<a title="blah">example</a>'
     )
 
 
@@ -501,8 +503,8 @@ def test_attributes_wildcard_callable():
     TAGS = ['a']
 
     assert (
-        clean(u'<a href="/foo" title="blah">example</a>', tags=TAGS, attributes=ATTRS) ==
-        u'<a title="blah">example</a>'
+        clean('<a href="/foo" title="blah">example</a>', tags=TAGS, attributes=ATTRS) ==
+        '<a title="blah">example</a>'
     )
 
 
@@ -519,12 +521,12 @@ def test_attributes_tag_callable():
     text = 'foo <img src="http://example.com" alt="blah"> baz'
     assert (
         clean(text, tags=TAGS, attributes=ATTRS) ==
-        u'foo <img> baz'
+        'foo <img> baz'
     )
     text = 'foo <img src="https://example.com" alt="blah"> baz'
     assert (
         clean(text, tags=TAGS, attributes=ATTRS) ==
-        u'foo <img src="https://example.com"> baz'
+        'foo <img src="https://example.com"> baz'
     )
 
 
@@ -536,8 +538,8 @@ def test_attributes_tag_list():
     TAGS = ['a']
 
     assert (
-        clean(u'<a href="/foo" title="blah">example</a>', tags=TAGS, attributes=ATTRS) ==
-        u'<a title="blah">example</a>'
+        clean('<a href="/foo" title="blah">example</a>', tags=TAGS, attributes=ATTRS) ==
+        '<a title="blah">example</a>'
     )
 
 
@@ -546,10 +548,10 @@ def test_attributes_list():
     ATTRS = ['title']
     TAGS = ['a']
 
-    text = u'<a href="/foo" title="blah">example</a>'
+    text = '<a href="/foo" title="blah">example</a>'
     assert (
         clean(text, tags=TAGS, attributes=ATTRS) ==
-        u'<a title="blah">example</a>'
+        '<a title="blah">example</a>'
     )
 
 
@@ -769,6 +771,57 @@ def test_nonexistent_namespace():
     assert clean('<d {c}>') == '&lt;d {c}&gt;'
 
 
+# tags that get content passed through (i.e. parsed with parseRCDataRawtext)
+_raw_tags = [
+    "title",
+    "textarea",
+    "script",
+    "style",
+    "noembed",
+    "noframes",
+    "iframe",
+    "xmp",
+]
+
+@pytest.mark.parametrize(
+    "raw_tag, data, expected",
+    [
+        (
+            raw_tag,
+            "<noscript><%s></noscript><img src=x onerror=alert(1) />" % raw_tag,
+            "<noscript>&lt;%s&gt;</noscript>&lt;img src=x onerror=alert(1) /&gt;" % raw_tag,
+        )
+        for raw_tag in _raw_tags
+    ],
+)
+def test_noscript_rawtag_(raw_tag, data, expected):
+    # refs: bug 1615315 / GHSA-q65m-pv3f-wr5r
+    assert clean(data, tags=["noscript", raw_tag]) == expected
+
+
+@pytest.mark.parametrize(
+    "namespace_tag, rc_data_element_tag, data, expected",
+    [
+        (
+            namespace_tag,
+            rc_data_element_tag,
+            "<%s><%s><img src=x onerror=alert(1)>" % (namespace_tag, rc_data_element_tag),
+            "<%s><%s>&lt;img src=x onerror=alert(1)&gt;</%s></%s>" % (namespace_tag, rc_data_element_tag, rc_data_element_tag, namespace_tag),
+        )
+        for namespace_tag in ["math", "svg"]
+        # https://dev.w3.org/html5/html-author/#rcdata-elements
+        # https://html.spec.whatwg.org/index.html#parsing-html-fragments
+        # in html5lib: 'style', 'script', 'xmp', 'iframe', 'noembed', 'noframes', and 'noscript'
+        for rc_data_element_tag in rcdataElements
+    ],
+)
+def test_namespace_rc_data_element_strip_false(namespace_tag, rc_data_element_tag, data, expected):
+    # refs: bug 1621692 / GHSA-m6xf-fq7q-8743
+    #
+    # browsers will pull the img out of the namespace and rc data tag resulting in XSS
+    assert clean(data, tags=[namespace_tag, rc_data_element_tag], strip=False) == expected
+
+
 def get_ids_and_tests():
     """Retrieves regression tests from data/ directory
 
@@ -783,10 +836,11 @@ def get_ids_and_tests():
     # Sort numerically which makes it easier to iterate through them
     tests.sort(key=lambda x: int(os.path.basename(x).split('.', 1)[0]))
 
-    testcases = [
-        (os.path.basename(fn), open(fn, 'r').read())
-        for fn in tests
-    ]
+    testcases = []
+    for fn in tests:
+        with open(fn) as fp:
+            data = fp.read()
+        testcases.append((os.path.basename(fn), data))
 
     return testcases
 
